//------------------------------------------------------------------------------------------------------------------------
// Class       : BP::CL_BP_PlayAudio
// Author      : BehemothProgrammer
// Description : Plays a sound when triggered
// OnTrigger   : Plays the sound
//
// Parameters  : (3) Max Regenerations -32768..32767	: Flags
//               (0) Target TID -32768..32767			: Sound ID
//               Radius									: Delay until sound plays
//------------------------------------------------------------------------------------------------------------------------
namespace BP
{
	enum PlayAudioFlags
	{
		PAF_REPEATABLE = (1 << 0),
		PAF_LOOPING = (1 << 1),
		PAF_CUSTOMSOUND = (1 << 2), //use custom sounds from BP_GameDefCustom actor def
		PAF_ONPLAYER = (1 << 3), //plays sound on playerActor
	}

	enum PlayAudioStates
	{
		PAS_IDLE = 0,
		PAS_DELAYING
	}

	class CL_BP_PlayAudio : CL_BP_ScriptObject
	{
		int flags;
		int soundID;
		float delay;
		//------------------------------------------------------------------------------------------------------------------------
		CL_BP_PlayAudio(kActor @actor)
		{
			super(actor);
			state = PAS_IDLE;
		}
		//------------------------------------------------------------------------------------------------------------------------
		void OnBeginLevel()
		{
			flags = self.SpawnParams(3);
			soundID = self.SpawnParams(0);
			delay = self.Radius();
			
			SetState(PAS_IDLE);
			
			self.Flags() &= ~AF_SOLID;
			self.Flags() |= AF_INVINCIBLE;
		}
		//------------------------------------------------------------------------------------------------------------------------
		void OnRestore()
		{
			self.Flags() |= AF_ACTIVATED;
		}
		//------------------------------------------------------------------------------------------------------------------------
		void OnTick() override
		{
			switch (state)
			{
				case PAS_IDLE:
				{
					if (flags & PAF_LOOPING != 0 && self.IsPersistentMarked())
						PlaySound();
					break;
				}
				case PAS_DELAYING:
				{
					if (BP::Time::Reached(stateEndTime))
					{
						PlaySound();
						SetState(PAS_IDLE);
					}
					break;
				}
			}
		}
		//------------------------------------------------------------------------------------------------------------------------
		void OnActivate()
		{
			if (flags & PAF_REPEATABLE != 0 && flags & PAF_LOOPING == 0)
			{
				self.Flags() &= ~AF_ACTIVATED;
				if (state != PAS_IDLE)
					return;
			}
			else
			{
				self.MarkPersistentBit(false);
			}
			
			if (delay > 0.0f)
				SetState(PAS_DELAYING);
			else
				PlaySound();
		}
		//------------------------------------------------------------------------------------------------------------------------
		void OnStateChange() override
		{
			switch (state)
			{
				case PAS_IDLE:
				{
					self.Flags() &= ~AF_ALWAYSACTIVE;
					break;
				}
				case PAS_DELAYING:
				{
					self.Flags() |= AF_ALWAYSACTIVE;
					self.Flags() &= ~AF_DORMANT;
					stateEndTime = BP::Time::FromNow(delay);
					break;
				}
			}
		}
		//------------------------------------------------------------------------------------------------------------------------
		void PlaySound()
		{
			if (soundID > 0)
			{
				kActor@ soundActor = (flags & PAF_ONPLAYER != 0) ? playerActor : self;
				if (flags & PAF_CUSTOMSOUND != 0)
					soundActor.PlaySound(BP::Game::GetCustomSound(soundID));
				else
					soundActor.PlaySoundWithLookupID(soundID);
			}
		}
		//------------------------------------------------------------------------------------------------------------------------
	};
}
