//------------------------------------------------------------------------------------------------------------------------
// Class       : BP::CL_BP_TriggerArea
// Author      : BehemothProgrammer
// Description : When the player enters the actors radius this can trigger other actors or map scripts.
// OnTrigger   : Toggles Enable/Disable the area. Is Enabled/Disabled is persistent.
//
// Parameters  : (7) Params 1 -128..127 				: Trigger Area Type. Not currently used.
//               (3) Max Regenerations -32768..32767	: Flags
//               (0) Target TID -32768..32767 			: Actors TID to trigger on touch. If is less than 1 this will not be used.
//               (2) Health -32768..32767				: Script to trigger on touch. If is less than 1 this will not be used.
//------------------------------------------------------------------------------------------------------------------------
namespace BP
{
	enum TRIGGERAREATypes
	{
		TAREA_TYPE_TRIGGER = 0
	}

	enum TRIGGERAREAFlags
	{
		TAREA_FLAG_STARTDISABLED = (1 << 0),
		TAREA_FLAG_GROUNDED = (1 << 1) //triggers only when the player is grounded
	}

	class CL_BP_TriggerArea : CL_BP_ScriptObject
	{
		int areaType;
		int flags;
		bool isActive;
		int triggerTID;
		int triggerScriptID;
		//------------------------------------------------------------------------------------------------------------------------
		CL_BP_TriggerArea(kActor @actor)
		{
			super(actor);
		}
		//------------------------------------------------------------------------------------------------------------------------
		void OnBeginLevel()
		{
			areaType = self.SpawnParams(7);
			flags = self.SpawnParams(3);
			triggerTID = self.SpawnParams(0);
			triggerScriptID = self.SpawnParams(2);

			self.Flags() &= ~AF_SOLID;
			self.Flags() |= AF_INVINCIBLE;
			
			//Set initial active
			bool startActive = !self.IsPersistentMarked();
			if (flags & TAREA_FLAG_STARTDISABLED != 0)
				startActive = !startActive;
			SetActive(startActive);
		}
		//------------------------------------------------------------------------------------------------------------------------
		void OnTouch(kActor @actor)
		{
			if (actor.Type() != AT_PLAYER) return;
			if (flags & TAREA_FLAG_GROUNDED != 0 && !actor.OnGround()) return;
			
			AreaTriggered();
		}
		//------------------------------------------------------------------------------------------------------------------------
		void OnActivate()
		{
			self.Flags() &= ~AF_ACTIVATED;
			BP::Actor::TogglePersistentBit(self);
			bool setActive = !self.IsPersistentMarked();
			if (flags & TAREA_FLAG_STARTDISABLED != 0)
				setActive = !setActive;
			SetActive(setActive);
		}
		//------------------------------------------------------------------------------------------------------------------------
		void SetActive(const bool active)
		{
			isActive = active;
			if (isActive)
			{
				self.Flags() |= AF_CANBETOUCHED;
			}
			else
			{
				self.Flags() &= ~AF_CANBETOUCHED;
			}
		}
		//------------------------------------------------------------------------------------------------------------------------
		void AreaTriggered()
		{
			if (triggerTID > 0)
				World.TriggerActorsByTID(self, triggerTID);
			if (triggerScriptID > 0)
				Game.CallDelayedMapScript(triggerScriptID, self, 0.0f);
		}
		//------------------------------------------------------------------------------------------------------------------------
	};
}
