//------------------------------------------------------------------------------------------------------------------------
// Class       : BP::CL_BP_QuakeSource
// Author      : BehemothProgrammer
// Description : Causes the players screen to shake when close to this actor. When time is over this actor will be removed.
// OnTrigger   : Nothing.
//------------------------------------------------------------------------------------------------------------------------
namespace BP
{
	class CL_BP_QuakeSource : CL_BP_ScriptObject
	{
		float duration;
		float currentTime;
		float velocity;
		float angle;
		//------------------------------------------------------------------------------------------------------------------------
		CL_BP_QuakeSource(kActor @actor)
		{
			super(actor);
			duration = 0.0f;
		}
		//------------------------------------------------------------------------------------------------------------------------
		void OnTick() override
		{
			if (self.AnimState().PlayingID() > 0 && duration <= 0.0f)
			{
				if (self.AnimState().CycleCompleted())
					self.Remove();
				return;
			}
			
			if (currentTime <= 0.0f)
			{
				self.Remove();
				return;
			}
			
			if ((PlayLoop.Ticks() & 3) == 0)
			{
				kPuppet@ target = Player.Actor();
				float power;
				float dist;
				float av, rv;
				float dx, dy;
			
				power = currentTime / duration;
				av = (Math::RandCFloat() * (angle * 0.5f)) * power;
				rv = (Math::RandCFloat() * (angle * 0.5f)) * power;
				
				dx = self.Origin().x - target.Origin().x;
				dy = self.Origin().y - target.Origin().y;
				
				dist = 1024.0f - Math::Sqrt(dx * dx + dy * dy);
				
				if(dist < 0.0f)
				{
					dist = 0.0f;
				}
				
				target.RecoilPitch() = (dist * av * 0.00390625f);
				target.Roll() = (dist * rv * 0.00390625f);
			}
			
			currentTime -= GAME_DELTA_TIME;
		}
		//------------------------------------------------------------------------------------------------------------------------
		void StartShake(const float v, const float a, const float d)
		{
			velocity = v;
			angle = a * 0.3333333432674408f;
			duration = Math::Fabs(d);
			currentTime = duration;
		}
		//------------------------------------------------------------------------------------------------------------------------
		void ScreenShake(kActor @instigator, const float w, const float x, const float y, const float z)
		{
			StartShake(1.125f * GAME_SCALE, -0.1048f, self.StepHeight());
		}
		//------------------------------------------------------------------------------------------------------------------------
		void CauseThump(kActor @instigator, const float w, const float x, const float y, const float z)
		{
			float d = self.StepHeight();
			float vel = d * 1.706666666666667f;
			float ang = -(Math::pi / (60.0f / d));
			StartShake(vel, ang, d);
		}
		//------------------------------------------------------------------------------------------------------------------------
	};
}
