//------------------------------------------------------------------------------------------------------------------------
// Class       : BP::CL_BP_ActorIterator
// Author      : BehemothProgrammer
// Description : Calls custom tick functions on each actor one at a time per tick. It is required that a onTick funcdef
//				 is assigned.
//------------------------------------------------------------------------------------------------------------------------
namespace BP
{
	funcdef bool BP_FUNCDEF_ACTORITERTICK(kActor@ actor); //return true to remove actor from actors array
	funcdef void BP_FUNCDEF_ACTORITEREMPTY();

	final class CL_BP_ActorIterator
	{
		array<kActor@> actors;
		int maxPerTick;
		int index;
		BP_FUNCDEF_ACTORITERTICK@ onTick;
		BP_FUNCDEF_ACTORITEREMPTY@ onEmpty; //Called when actors length becames 0
		//------------------------------------------------------------------------------------------------------------------------
		CL_BP_ActorIterator()
		{
			index = 0;
			maxPerTick = 1;
		}
		//------------------------------------------------------------------------------------------------------------------------
		void Setup(const int perTick, BP_FUNCDEF_ACTORITERTICK@ tickFunc, BP_FUNCDEF_ACTORITEREMPTY@ emptyFunc = null)
		{
			maxPerTick = perTick;
			@onTick = tickFunc;
			@onEmpty = emptyFunc;
		}
		//------------------------------------------------------------------------------------------------------------------------
		void AddActor(kActor@ actor)
		{
			actors.insertLast(actor);
		}
		//------------------------------------------------------------------------------------------------------------------------
		bool IsEmpty()
		{
			return actors.length() == 0;
		}
		//------------------------------------------------------------------------------------------------------------------------
		uint ActorLength()
		{
			return actors.length();
		}
		//------------------------------------------------------------------------------------------------------------------------
		void CallOnEmpty()
		{
			if (onEmpty !is null)
				onEmpty();
		}
		//------------------------------------------------------------------------------------------------------------------------
		void Clear()
		{
			index = 0;
			uint startActorsLength = actors.length();
			if (startActorsLength > 0)
			{
				for (uint i = 0; i < startActorsLength; i++)
				{
					if (BP::Actor::Exists(actors[i]))
					{
						actors[i].Remove();
					}
				}
				actors.resize(0);
				CallOnEmpty();
			}
		}
		//------------------------------------------------------------------------------------------------------------------------
		void Tick()
		{
			int actorsTicked = 0;
			kActor@ actor;
			bool remove = false;
			int startActorsLength = int(actors.length());
			int maxActorsToTick = BP::Math::Min(startActorsLength, maxPerTick);
			
			while (actors.length() > 0 && actorsTicked < maxActorsToTick)
			{
				@actor = actors[index];
				remove = onTick(actor);
				actorsTicked++;
				
				if (remove)
				{
					actors.removeAt(index);
					index = BP::Array::NextIndex(actors.length(), index, 0);
				}
				else
				{
					index = BP::Array::NextIndex(actors.length(), index);
				}
			}
			
			if (startActorsLength > 0 && actors.length() == 0)
				CallOnEmpty();
		}
		//------------------------------------------------------------------------------------------------------------------------
	};
}
