namespace BP { namespace Game
{
	array<kStr> fxNames;
	array<kStr> customFxNames;
	array<kStr> soundNames;
	kActor@ raySrc;
	kActor@ rayDest;
	bool defLoaded;
	
	array<array<kStr>> localText(BP::LANGUAGE_COUNT);
	//------------------------------------------------------------------------------------------------------------------------
	// clipFlags: Specify what flags to not use (CF_NOCLIPACTORS means it will hit actors, 0 = everything)
	// Returns true if an intersection occured. CModel will store the result of the RayTrace.
	//------------------------------------------------------------------------------------------------------------------------
	bool RayTrace(const kVec3 &in from, const kVec3 &in to, uint clipFlags = 0)
	{
		if (!BP::Actor::Exists(raySrc))
			@raySrc = BP::Actor::Spawn(AT_RAYTRACE, from);
		if (!BP::Actor::Exists(rayDest))
			@rayDest = BP::Actor::Spawn(AT_RAYTRACE, to);
			
		raySrc.CanSee(rayDest, clipFlags);
		return CModel.Fraction() < 1.0f;
	}
	//------------------------------------------------------------------------------------------------------------------------
	void AddShake(kActor @actor, const float duration, const bool thump = false)
	{
		AddShake(actor.Origin(), actor.SectorIndex(), duration, thump);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void AddShake(const kVec3 &in point, const int sectorID, const float duration, const bool thump = false)
	{
		kActor@ actor = BP::Actor::Spawn(ACTOR_NAME_QUAKESOURCE, point, 0.0f, sectorID);
		actor.AnimState().Set(thump ? QUAKE_ANIM_THUMP : QUAKE_ANIM_SHAKE, 4.0f, 0);
		actor.StepHeight() = duration;
	}
	//------------------------------------------------------------------------------------------------------------------------
	int GetCounter(const int index)
	{
		return BP::GameVar::GetIntFromStringArray(GV_KEY_COUNTERS, index, MAX_COUNTERS);
	}
	//------------------------------------------------------------------------------------------------------------------------
	int IncreaseCounter(const int index, const int amount = 1)
	{
		return BP::GameVar::AddIntToStringArray(GV_KEY_COUNTERS, index, MAX_COUNTERS, amount);
	}
	//------------------------------------------------------------------------------------------------------------------------
	int SetCounterBitMask(const int index, const int bitMask, const bool on)
	{
		return BP::GameVar::SetIntBitMaskToStringArray(GV_KEY_COUNTERS, index, MAX_COUNTERS, bitMask, on);
	}
	//------------------------------------------------------------------------------------------------------------------------
	kStr GetFx(const int index)
	{
		if (HasFx(index))
			return fxNames[index - 1];
		
		return "";
	}
	//------------------------------------------------------------------------------------------------------------------------
	bool HasFx(const int index)
	{
		return BP::Array::IndexInRange(fxNames.length(), index - 1);
	}
	//------------------------------------------------------------------------------------------------------------------------
	kStr GetCustomFx(const int index)
	{
		if (HasCustomFx(index))
			return customFxNames[index - 1];
		
		return "";
	}
	//------------------------------------------------------------------------------------------------------------------------
	bool HasCustomFx(const int index)
	{
		return BP::Array::IndexInRange(customFxNames.length(), index - 1);
	}
	//------------------------------------------------------------------------------------------------------------------------
	kStr GetCustomSound(const int index)
	{
		if (HasCustomSound(index))
			return soundNames[index - 1];
		
		return "";
	}
	//------------------------------------------------------------------------------------------------------------------------
	bool HasCustomSound(const int index)
	{
		return BP::Array::IndexInRange(soundNames.length(), index - 1);
	}
	//------------------------------------------------------------------------------------------------------------------------
	int GetCurrentLanguage()
	{
		int language = BP::CVar::GetInt(BP::CVAR_LANGUAGE);
		if (language < 0 || language >= LANGUAGE_COUNT)
			language = LANGUAGE_ENGLISH;
		
		return language;
	}
	//------------------------------------------------------------------------------------------------------------------------
	kStr GetLocalText(const int index)
	{
		return GetLocalText(index, GetCurrentLanguage());
	}
	//------------------------------------------------------------------------------------------------------------------------
	kStr GetLocalText(const int index, const int language)
	{
		if (HasLocalText(index, language))
			return localText[language][index - 1];
		
		if (language != LANGUAGE_ENGLISH)
		{
			//try getting the english text
			if (HasLocalText(index, LANGUAGE_ENGLISH))
				return localText[LANGUAGE_ENGLISH][index - 1];
		}
		
		return "";
	}
	//------------------------------------------------------------------------------------------------------------------------
	bool HasLocalText(const int index)
	{
		return HasLocalText(index, GetCurrentLanguage());
	}
	//------------------------------------------------------------------------------------------------------------------------
	bool HasLocalText(const int index, const int language)
	{
		if (!BP::Array::IndexInRange(localText.length(), language))
			return false;
		
		return BP::Array::IndexInRange(localText[language].length(), index - 1);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void LoadDef()
	{
		if (!defLoaded)
		{
			defLoaded = true;
			kActor@ actor = BP::Actor::Spawn(ACTOR_NAME_GAMEDEF);
			if (BP::Actor::Exists(actor))
			{
				BP::String::asciiChars[35] = BP::Actor::GetDefString(actor, DEF_STRING_HASH);
				BP::String::asciiChars[36] = BP::Actor::GetDefString(actor, DEF_STRING_DOLLAR);
				fxNames = BP::Actor::GetDefStringArray(actor, "fx");
				
				actor.Remove();
			}
			
			@actor = BP::Actor::Spawn(ACTOR_NAME_GAMEDEFCUSTOM);
			if (BP::Actor::Exists(actor))
			{
				customFxNames = BP::Actor::GetDefStringArray(actor, "fx");
				soundNames = BP::Actor::GetDefStringArray(actor, "sound");
				localText[LANGUAGE_ENGLISH] = BP::Actor::GetDefStringArray(actor, "english_text");
				localText[LANGUAGE_FRENCH] = BP::Actor::GetDefStringArray(actor, "french_text");
				localText[LANGUAGE_GERMAN] = BP::Actor::GetDefStringArray(actor, "german_text");
				localText[LANGUAGE_ITALIAN] = BP::Actor::GetDefStringArray(actor, "italian_text");
				localText[LANGUAGE_SPANISH] = BP::Actor::GetDefStringArray(actor, "spanish_text");
				
				actor.Remove();
			}
		}
	}
	//------------------------------------------------------------------------------------------------------------------------
}}
