namespace BP { namespace HUD
{
	const int MAX_MESSAGES = 7; //Must be 7 or less
	
	enum MessageVertAlignments
	{
		MSG_CENTER = 0,
		MSG_TOP,
		MSG_BOTTOM,
	}
	//------------------------------------------------------------------------------------------------------------------------
	void ShowMessage(const kStr &in msg1, const int time, const int alignment = BP::HUD::MSG_CENTER)
	{
		ShowMessage(array<kStr>={msg1}, time, alignment);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void ShowMessage(const kStr &in msg1, const kStr &in msg2, const int time, const int alignment = BP::HUD::MSG_CENTER)
	{
		ShowMessage(array<kStr>={msg1, msg2}, time, alignment);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void ShowMessage(const kStr &in msg1, const kStr &in msg2, const kStr &in msg3, const int time, const int alignment = BP::HUD::MSG_CENTER)
	{
		ShowMessage(array<kStr>={msg1, msg2, msg3}, time, alignment);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void ShowMessage(const kStr &in msg1, const kStr &in msg2, const kStr &in msg3, const kStr &in msg4, const int time, const int alignment = BP::HUD::MSG_CENTER)
	{
		ShowMessage(array<kStr>={msg1, msg2, msg3, msg4}, time, alignment);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void ShowMessage(const kStr &in msg1, const kStr &in msg2, const kStr &in msg3, const kStr &in msg4,
						const kStr &in msg5, const int time, const int alignment = BP::HUD::MSG_CENTER)
	{
		ShowMessage(array<kStr>={msg1, msg2, msg3, msg4, msg5}, time, alignment);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void ShowMessage(const kStr &in msg1, const kStr &in msg2, const kStr &in msg3, const kStr &in msg4,
						const kStr &in msg5, const kStr &in msg6, const int time, const int alignment = BP::HUD::MSG_CENTER)
	{
		ShowMessage(array<kStr>={msg1, msg2, msg3, msg4, msg5, msg6}, time, alignment);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void ShowMessage(const kStr &in msg1, const kStr &in msg2, const kStr &in msg3, const kStr &in msg4,
						const kStr &in msg5, const kStr &in msg6, const kStr &in msg7, const int time, const int alignment = BP::HUD::MSG_CENTER)
	{
		ShowMessage(array<kStr>={msg1, msg2, msg3, msg4, msg5, msg6, msg7}, time, alignment);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void ShowMessage(array<kStr> &lines, const int time, const int alignment = BP::HUD::MSG_CENTER)
	{
		int linesLength = int(lines.length());
		int linesNeeded = MAX_MESSAGES - linesLength;
		
		if (linesNeeded < 0)
		{
			lines.resize(MAX_MESSAGES);
		}
		else
		{
			switch (alignment)
			{
				case MSG_CENTER:
				{
					int halfLinesNeeded = linesNeeded / 2;
					for (int i = 0; i < halfLinesNeeded; i++)
						lines.insertAt(0, "");
					
					int bottomLinesNeeded = halfLinesNeeded;
					if (linesNeeded % 2 == 1)
						bottomLinesNeeded += 1;
					for (int i = 0; i < bottomLinesNeeded; i++)
						lines.insertLast("");
					break;
				}
				case MSG_TOP:
				{
					for (int i = 0; i < linesNeeded; i++)
						lines.insertLast("");
					break;
				}
				case MSG_BOTTOM:
				{
					for (int i = 0; i < linesNeeded; i++)
						lines.insertAt(0, "");
					break;
				}
			}
		}
		
		for (int i = 0; i < MAX_MESSAGES; i++)
		{
			Game.PrintLine(lines[(MAX_MESSAGES - 1) - i], i, time);
		}
	}
	//------------------------------------------------------------------------------------------------------------------------
	void ShowHelp(array<kStr> &lines, const int time)
	{
		kStr msg;
		for (uint i = 0; i < lines.length(); i++)
			msg += lines[i];
		ShowHelp(msg, time);
	}
	//------------------------------------------------------------------------------------------------------------------------
	// time: 60 = 1 second. Use \n to create newlines
	//------------------------------------------------------------------------------------------------------------------------
	void ShowHelp(const kStr &in msg, const int time)
	{
		BP::GameVar::SetString(GV_KEY_HUDHELP, msg);
		BP::GameVar::SetInt(GV_KEY_HUDHELPTIME, time);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void ClearMessages()
	{
		for (int i = 0; i < MAX_MESSAGES; i++)
		{
			Game.PrintLine("", i, 10);
		}
	}
	//------------------------------------------------------------------------------------------------------------------------
}}
