namespace BP { namespace Music
{
	enum MusicFlags
	{
		MF_ISPLAYING = (1 << 0),
		MF_LISTUPDATE = (1 << 1),
		MF_STOPONNEXTMAP = (1 << 2),
		MF_DISABLE = (1 << 3),
	}

	enum MusicPlayOrder
	{
		MUSIC_PO_SEQUENTIAL = 0,
		MUSIC_PO_RANDOM = 1,
		MUSIC_PO_RANDOMAFTERFIRST = 2 //plays the first track then random after that
	}

	//------------------------------------------------------------------------------------------------------------------------
	bool IsPlaying()
	{
		return BP::GameVar::GetIntFlag(GV_KEY_MUSICFLAGS, MF_ISPLAYING);
	}
	//------------------------------------------------------------------------------------------------------------------------
	bool IsDisabled()
	{
		return BP::GameVar::GetIntFlag(GV_KEY_MUSICFLAGS, MF_DISABLE);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void Play(const kStr &in path, const bool stopOnNewMap = true)
	{
		kStr data = "" + (stopOnNewMap ? Game.GetCurrentMapID() : -2) + "|0|" + path + "|-1";
		SetEventData(data);
		BP::GameVar::SetIntFlag(GV_KEY_MUSICFLAGS, MF_ISPLAYING | MF_LISTUPDATE);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void PlayList(const array<kStr> &in paths, const array<float> &in durations, const int playOrder = MUSIC_PO_SEQUENTIAL, const bool stopOnNewMap = true)
	{
		kStr data = "" + (stopOnNewMap ? Game.GetCurrentMapID() : -2) + "|" + playOrder;
		for (uint i = 0; i < paths.length(); i++)
			data += "|" + paths[i] + "|" + durations[i];
		
		SetEventData(data);
		BP::GameVar::SetIntFlag(GV_KEY_MUSICFLAGS, MF_ISPLAYING | MF_LISTUPDATE);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void Stop()
	{
		BP::GameVar::SetIntFlag(GV_KEY_MUSICFLAGS, MF_ISPLAYING, false);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void Disable()
	{
		BP::GameVar::SetIntFlag(GV_KEY_MUSICFLAGS, MF_DISABLE);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void Enable()
	{
		BP::GameVar::SetIntFlag(GV_KEY_MUSICFLAGS, MF_DISABLE, false);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void StopOnNextMap()
	{
		BP::GameVar::SetIntFlag(GV_KEY_MUSICFLAGS, MF_STOPONNEXTMAP);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void Resume()
	{
		BP::GameVar::SetIntFlag(GV_KEY_MUSICFLAGS, MF_ISPLAYING);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void SetEventData(const kStr &in data)
	{
		BP::GameVar::SetString(GV_KEY_MUSICDATA, data);
	}
	//------------------------------------------------------------------------------------------------------------------------
	bool GetEventData(array<kStr> &out trackPaths, array<float> &out trackDurations, int &out playOrder, int &out mapID)
	{
		array<kStr> dataAry = BP::String::Split(BP::GameVar::GetString(GV_KEY_MUSICDATA), "|");
		if (dataAry.length() == 0)
			return false;
		
		mapID = dataAry[0].Atoi();
		playOrder = dataAry[1].Atoi();
		trackPaths.resize(0);
		trackDurations.resize(0);
		for (uint i = 2; i < dataAry.length(); i+=2)
		{
			trackPaths.insertLast(dataAry[i]);
			trackDurations.insertLast(dataAry[i+1].Atof());
		}
		
		return true;
	}
	//------------------------------------------------------------------------------------------------------------------------
}}
