namespace BP { namespace Player
{
	const float MELEE_RADIUS = 5.75f * GAME_SCALE;

	//------------------------------------------------------------------------------------------------------------------------
	kVec3 MeleePos()
	{
		kActor@ actor = Player.Actor().CastToActor();
		kVec3 pos = actor.Origin() + (kVec3(0.0f, MELEE_RADIUS, 0.0f) * actor.Rotation());
		pos.z += (GAME_SCALE * 3);
		
		return pos;
	}
	//------------------------------------------------------------------------------------------------------------------------
	void SetPosition(kActor@ actorLocation, const float yaw = 0.0f, const float pitch = 0.0f, const bool floorHeight = true)
	{
		kActor@ actor = Player.Actor().CastToActor();
		BP::Actor::SetPosition(actor, actorLocation, floorHeight);
		actor.Yaw() = yaw;
		actor.Pitch() = pitch;
		actor.Roll() = 0.0f;
		Player.Actor().RecoilPitch() = 0.0f;
		actor.Velocity() = Math::vecZero;
		actor.Movement() = Math::vecZero;
	}
	//------------------------------------------------------------------------------------------------------------------------
	void PlayAnim(const int animID)
	{
		int animFlags = 0;
		float animSpeed = 4.0f;
		switch (animID)
		{
			case anim_player_panic:
				animSpeed = 10.0f;
				animFlags = ANF_LOOP | ANF_ROOTMOTION;
				break;

			case anim_player_exit: //jump out of tunnel with explosion
			case anim_campaingerRage: //campaigner death
				animSpeed = 8.0f;
				animFlags = ANF_ROOTMOTION;
				break;

			case anim_player_idle:
			case anim_aiStanding: //boss victory
			{
				animSpeed = 8.0f;
				animFlags = ANF_LOOP;
				break;
			}
			case anim_player_run: //with explosions
			{
				animSpeed = 5.5f;
				animFlags = ANF_LOOP | ANF_ROOTMOTION;
				break;
			}
			case anim_player_jump: //freeze jump fall animation
			case 36: // Death Pit Falling
			{
				animSpeed = 4.0f;
				animFlags = ANF_LOOP;
				break;
			}
			case anim_player_surface: //rises from water and looks back up
			case anim_activate: //mantis death
			{
				animSpeed = 4.0f;
				animFlags = 0;
				break;
			}
			case anim_aiDeathStand: //normal death
			case anim_turokIsNom: //falls to ground blocking
			case anim_aiSwim8: //underwater death
			case anim_aiTeleportIn: //revive
			case anim_special_event01: //intro waits with bow and then fires it
			case anim_player_acquire_key:
			{
				animSpeed = 8.0f;
				animFlags = 0;
				break;
			}
		}

		Player.Actor().AnimState().Set(animID, animSpeed, animFlags);
	}
	//------------------------------------------------------------------------------------------------------------------------
}}
