namespace BP { namespace String
{
	array<kStr> asciiChars = {
		"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
		"", "", ' ', '!', '"', "", "", '%', '&', '\'', '(', ')', '*', '+', ',', '-', ".", "/", '0', '1', '2', '3', '4', '5', '6',
		'7', '8', '9', ':', ';', '<', '=', '>', '?', '@', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N',
		'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '[', '\\', ']', '^', '_', '`', 'a', 'b', 'c', 'd', 'e', 'f',
		'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '{', '|', '}', '~', ""
	};

	const kStr STRINGLENGTH_KEY = ";L;";
	//------------------------------------------------------------------------------------------------------------------------
	// If s already contains STRINGLENGTH_KEY then this will return an incorrect length
	//------------------------------------------------------------------------------------------------------------------------
	int Length(kStr &in s)
	{
		return (s + STRINGLENGTH_KEY).IndexOf(STRINGLENGTH_KEY);
	}
	//------------------------------------------------------------------------------------------------------------------------
	bool IsEmpty(const kStr &in s)
	{
		return s.Hash() == 0;
	}
	//------------------------------------------------------------------------------------------------------------------------
	// uint to string
	//------------------------------------------------------------------------------------------------------------------------
	kStr UInt(const uint value)
	{
		if (value & BP::Math::UBIT31 == 0)
			return "" + value;
		kStr result;
		array<int> intDigits = BP::Math::Digits(value, false);
		for (uint i = 0; i < intDigits.length(); i++)
			result = result + intDigits[i];
		return result;
	}
	//------------------------------------------------------------------------------------------------------------------------
	// destLength: the length of the desired string
	// padChar: the character to put in the padding.
	// Example: PadLeft("hello", 7, "0")  // "00hello"
	//------------------------------------------------------------------------------------------------------------------------
	kStr PadLeft(const kStr &in s, const int destLength, kStr &in padChar)
	{
		kStr result = s;
		int sLength = Length(s);
		for (int i = sLength; i < destLength; i++)
			result = padChar + result;
		
		return result;
	}
	//------------------------------------------------------------------------------------------------------------------------
	// destLength: the length of the desired string
	// padChar: the character to put in the padding.
	// Example: PadRight("hello", 7, "0")  // "hello00"
	//------------------------------------------------------------------------------------------------------------------------
	kStr PadRight(const kStr &in s, const int destLength, kStr &in padChar)
	{
		kStr result = s;
		int sLength = Length(s);
		for (int i = sLength; i < destLength; i++)
			result += padChar;
		
		return result;
	}
	//------------------------------------------------------------------------------------------------------------------------
	kStr TimeFormatMMSS(const float time)
	{
		int mins = BP::Math::Min(int(time / 60.0f), 99);
		int sec = int(time % 60.0f);
		return "" + mins + ":" + PadLeft("" + sec, 2, "0");
	}
	//------------------------------------------------------------------------------------------------------------------------
	array<int> SplitInt(const kStr &in s, kStr &in c)
	{
		array<kStr> entries = Split(s, c);
		array<int> result;
		for (int i = 0; i < int(entries.length()); i++)
		{
			result.insertLast(entries[i].Atoi());
		}
		return result;
	}
	//------------------------------------------------------------------------------------------------------------------------
	array<kStr> Split(kStr &in s, kStr &in c)
	{
		int length = Length(s);
		array<kStr> result;
		if (length > 0)
		{
			int sepLength = Length(c);
			int sepMatchIndex = 0;
			int8 sepCharID = c[sepMatchIndex];
			kStr entry = "";
			for (int i = 0; i < length; i++)
			{
				int8 charID = s[i];
				if (charID == sepCharID)
				{
					sepMatchIndex++;
					if (sepMatchIndex >= sepLength)
					{
						sepMatchIndex = 0;
						sepCharID = c[sepMatchIndex];
						result.insertLast(entry);
						entry = "";
						continue;
					}
					sepCharID = c[sepMatchIndex];
				}
				else
				{
					sepMatchIndex = 0;
					sepCharID = c[sepMatchIndex];
					entry += asciiChars[charID];
				}
			}
			result.insertLast(entry);
		}
		return result;
	}
	//------------------------------------------------------------------------------------------------------------------------
	kStr CharAt(kStr &in s, int index)
	{
		int8 c = s[index];
		return c >= 0 ? asciiChars[c] : "";
	}
	//------------------------------------------------------------------------------------------------------------------------
}}
