//------------------------------------------------------------------------------------------------------------------------
// Class       : Q::CL_Q_FlyMantis
// Author      : BehemothProgrammer
// Description : Flying mantis enemy that spits acid and floats up and down near ground level. Never goes below water level.
// OnTrigger   : 
//
// Parameters  : (6) Params 2 -128..127					: Spawn Flags
//
//------------------------------------------------------------------------------------------------------------------------
namespace Q
{
	enum FlyMantisFlags
	{
		FMF_STARTPOSMIN = (1 << 0), //Use start position z as the ground height value
	}

	const float FLYMANTIS_FLY_HEIGHT_MIN = 48.0f;
	const float FLYMANTIS_FLY_HEIGHT_SCALE = GAME_SCALE * 3.0f;
	const float FLYMANTIS_FLOAT_SPEED = 2.0f;
	const float FLYMANTIS_FLOAT_GROUND_OFFSET = 1.0f;

	final class CL_Q_FlyMantis : CL_Q_Enemy
	{
		bool wasDead;
		float flyHeight;
		float targetOriginZ;
		float flyHeightAngle;
		int spawnFlags;
		float spawnPosZ;
		//------------------------------------------------------------------------------------------------------------------------
		CL_Q_FlyMantis(kActor @actor)
		{
			super(actor);
			flyHeightAngle = Math::RandRange(-90.0f, 90.0f);
			self.Gravity() = 0.0f;
			spawnPosZ = self.Origin().z + FLYMANTIS_FLOAT_GROUND_OFFSET;
		}
		//------------------------------------------------------------------------------------------------------------------------
		void OnBeginLevel()
		{
			spawnFlags = self.SpawnParams(6);
		}
		//------------------------------------------------------------------------------------------------------------------------
		void OnTick() override
		{
			CL_Q_Enemy::OnTick();
			
			if (BP::Actor::IsHidden(self))
				return;
			
			bool isDead = BP::Actor::IsDead(self);
			if (wasDead != isDead)
			{
				wasDead = isDead;
				if (isDead)
				{
					self.Gravity() = 0.5f;
				}
			}
			
			bool inWaterSector = (World.GetAreaFlags(self.AreaID()) & AAF_WATER) != 0;
			
			if (isDead)
			{
				if (inWaterSector && self.Origin().z <= self.GetWaterHeight())
				{
					self.Origin().z = self.GetWaterHeight() + FLYMANTIS_FLOAT_GROUND_OFFSET;
					self.Gravity() = 0.0f;
				}
			}
			else
			{
				self.Velocity() = Math::vecZero;
				
				flyHeight = FLYMANTIS_FLY_HEIGHT_MIN + (Math::Sin(flyHeightAngle) * FLYMANTIS_FLY_HEIGHT_SCALE);
				flyHeightAngle += BP::Math::ONEDEGREERAD * FLYMANTIS_FLOAT_SPEED;
				if (flyHeightAngle >= 360.0f)
					flyHeightAngle -= 360.0f;
				
				float groundHeight = self.FloorHeight() + FLYMANTIS_FLOAT_GROUND_OFFSET;
				if (inWaterSector)
					groundHeight = BP::Math::Max(self.FloorHeight(), self.GetWaterHeight()) + FLYMANTIS_FLOAT_GROUND_OFFSET;
				
				if (spawnFlags & FMF_STARTPOSMIN != 0 && groundHeight < spawnPosZ)
				{
					groundHeight = spawnPosZ;
				}

				targetOriginZ = BP::Math::Max(groundHeight + flyHeight, groundHeight);
				self.Origin().z = BP::Math::SmoothStepFloat(self.Origin().z, targetOriginZ, 0.1f);
				
				if (inWaterSector && self.Origin().z <= self.GetWaterHeight())
				{
					self.Origin().z = self.GetWaterHeight() + FLYMANTIS_FLOAT_GROUND_OFFSET;
				}
			}
		}
		//------------------------------------------------------------------------------------------------------------------------
		void SetBitMask1(kActor @instigator, const float w, const float x, const float y, const float z)
		{
			
		}
		//------------------------------------------------------------------------------------------------------------------------
		void SpitMantisAcid(kActor @instigator, const float w, const float x, const float y, const float z)
		{
			kActor@ target = self.GetTarget();
			if (target is null)
				return;
			
			if (self.AnimState().PlayingID() != anim_aiStanding && self.AnimState().PlayingID() != anim_aiRunning)
				return;
			
			if (!self.RandomDecision(5) || !self.CanSee(target))
				return;
			
			self.SpawnProjectile("fx/quake/flying_mantis_acid.kfx", kVec3(x, y, z), target.Origin(), Math::Deg2Rad(45.0f));
		}
		//------------------------------------------------------------------------------------------------------------------------
	};
}
