//------------------------------------------------------------------------------------------------------------------------
// Class       : Q::CL_Q_Zombie
// Author      : BehemothProgrammer
// Description : Enemies that come back to life.
// OnTrigger   : 
//
// Parameters  : 
//
//------------------------------------------------------------------------------------------------------------------------
namespace Q
{
	const float ZOMBIE_REGEN_TIME = 5.0; //time after death animation to regen
	const int ZOMBIE_GIB_HEALTH = -30; //health until real death with non explosives
	const int ZOMBIE_GIB_EXPLODEHEALTH = 0; //health until real death with explosives

	class CL_Q_Zombie : TurokGrunt
	{
		float reviveTime;
		bool reviving;
		bool nextOnDamageIsDeath;
		//------------------------------------------------------------------------------------------------------------------------
		CL_Q_Zombie(kActor @actor)
		{
			super(actor);
		}
		//------------------------------------------------------------------------------------------------------------------------
		void OnTick() override
		{
			TurokGrunt::OnTick();
			
			if (BP::Actor::IsHidden(self) || !BP::Actor::Exists(self))
				return;
			
			bool isDead = BP::Actor::IsDead(self);
			if (isDead)
			{
				if (reviving && self.AnimState().CycleCompleted() && BP::Time::Reached(reviveTime))
				{
					reviving = false;
					self.SpawnFx("fx/regeneration_start.kfx", Math::vecZero);
					self.Flags() &= ~AF_DEAD;
					self.CastToAI().AIFlags() |= BP::AIF_REGENDELAY2;
					self.Flags() &= ~AF_ALWAYSACTIVE;
				}
			}
		}
		//------------------------------------------------------------------------------------------------------------------------
		void OnDeath(kActor @killer, kDictMem @damageDef)
		{
			nextOnDamageIsDeath = true;
		}
		//------------------------------------------------------------------------------------------------------------------------
		void OnDamage(kActor @instigator, kDictMem @damageDef, const int damage)
		{
			if (nextOnDamageIsDeath)
			{
				int hp = self.Health() - damage;
				bool explodeHit;
				if (damageDef !is null)
					damageDef.GetBool("bExplode", explodeHit);
				int gibHealth = explodeHit ? ZOMBIE_GIB_EXPLODEHEALTH : ZOMBIE_GIB_HEALTH;
				
				if (damageDef !is null && hp <= gibHealth)
				{
					//Enemy Death
					BP::GameVar::SetInt(GV_KEY_MAP_KILLS, BP::GameVar::GetInt(GV_KEY_MAP_KILLS) + 1);
					self.ClipFlags() |= CF_DROPOFF;

					m_bDroppedItem = false;
					bool accelHit;
					if (damageDef.GetBool("bAccelerator", accelHit) && accelHit)
					{
						self.AnimState().flags |= (ANF_PAUSED|ANF_NOINTERRUPT);
						m_deathFreezeTime = 3.0f;
						self.RunFxEvent("Enemy_Freeze");
						Game.SpawnFx("fx/freeze_start.kfx", self.Origin(), self.SectorIndex());
					}
					else
					{
						//Drop Gibs
						array<kStr> gibNames = {ACTOR_NAME_GIB1, ACTOR_NAME_GIB2, ACTOR_NAME_GIB3, ACTOR_NAME_GIB4, ACTOR_NAME_GIB5, ACTOR_NAME_GIB6, ACTOR_NAME_GIB7, ACTOR_NAME_GIB8};
						kVec3 gibPos = self.Origin();
						gibPos.z += self.Height() * 0.5f;
						int gibCount = 4 + Math::RandMax(3);
						for (int i = 0; i < gibCount; i++)
						{
							int gibIndex = BP::Array::RandomIndex(gibNames.length());
							Game.SpawnFx(PATH_FX_GIBSPAWN, gibPos, self.SectorIndex());
							TossGib(gibNames[gibIndex], gibPos.x, gibPos.y, gibPos.z);
						}
						
						self.Remove();
					}
				}
				else
				{
					//Revive
					reviveTime = BP::Time::FromNow(ZOMBIE_REGEN_TIME);
					reviving = true;
					self.MarkPersistentBit(true);
					self.Flags() |= AF_ALWAYSACTIVE;
				}
			}
			
			nextOnDamageIsDeath = false;
		}
		//------------------------------------------------------------------------------------------------------------------------
	};
}
