#include "scripts/Q/Q_Common_Map.cpp"

enum EnumMapTags
{
	TAG_CAMPAIGNER_DUMMY_HEALTH = 1008,
	TAG_PILLAR_LEFT = 1010,
	TAG_PILLAR_RIGHT = 1011,
	TAG_CAMERA_MAPEXIT_POS = 1014,
	TAG_CAMERA_MAPEXIT_LOOK = 1015,
	TAG_ITEM_RUNE = 1016,
	TAG_CAMPAIGNER_WP_START = 1018,
	TAG_CAMPAIGNER_WP_FINISH = 1030,
	TAG_CAMPAIGNER_FUSION_CANNON = 1032,
	TAG_SHOCK_POINT = 1033,
	TAG_SHOW_ENEMIES2 = 1035,
	TAG_SHOW_ENEMIES3 = 1036,
	TAG_SHOW_ENEMIES4 = 1037,
	TAG_SHOW_ENEMIES5 = 1038,
	TAG_SHOW_ENEMIES6 = 1039,
	TAG_VICTORYGIB1 = 1040,
	TAG_VICTORYGIB_FINAL = 1046,
	TAG_AI_CAMPAIGNER = 1500,
}

//TIDs
//2605: Warp Start
//1000: Start Save Game EventLink
//1001: SectorModifier Turn on SaveGame Flag at start
//1002: Center Switch
//1003: Left Switch
//1004: Right Switch
//1005: Center Switch EventLink
//1006: Left Switch EventLink
//1007: Right Switch EventLink
//1008: Campaigner Dummy Health Actor
//1009: Door Bar, Stairs, SectorModifiers
//1010: Left Pillar
//1011: Right Pillar
//1012: Elevator, SectorModifier Blocker
//1013: Elevator EventLink
//1014: Map Exit Camera Pos
//1015: Map Exit Camera Look Target
//1016: Inventory 2 - Rune Piece
//1017: Rune Piece EventLink, Message - You got the Rune!, Actions: Show Enemy Group 1
//1018-1030: Campaigner Teleport Points
//1031: EventLink for TID 1009
//1032: Campaigners Fusion Cannon
//1033: Shock Fx Pos
//1034: Respawning pickups
//1035: Actions: Show Enemy Group 2 (Fight Camp 1)
//1036: Actions: Show Enemy Group 3 (Shield is up 2)
//1037: Actions: Show Enemy Group 4 (Fight Camp 2)
//1038: Actions: Show Enemy Group 5 (Shield is up 3)
//1039: Actions: Show Enemy Group 6 (Fight Camp 3)
//1040-1047: Victory Gib Positions
//1048: 

//1500-1528: Enemies


const float MAP_CAMPAIGNER_FORCE_TELEPORT_TIME = 20.0f; //how long the campaigner hasn't teleported and will be forced to teleport
const float MAP_CAMPAIGNER_NO_SEE_TELEPORT_TIME = 4.0f; //how long the campaigner hasn't seen the player for and then will teleport
const float MAP_CAMPAIGNER_TELEPORT_TIME = 1.0f; //when forced to teleport how long until will actually change position/do fx

const float MAP_CFC_DOWN_Z = -732.159973f;
const float MAP_CFC_DOWNLERP_Z = -640.0f;
const float MAP_CFC_UP_Z = 332.799988f;
const float MAP_CFC_MOVETIME = 5.0f;
const float MAP_CFC_EXPLODETIME = 5.0f;
const float MAP_PILLAR_SHOCKTIME = 2.0f;
const float MAP_PILLAR_ANIMSPEED = 2.5f;
const float MAP_PILLAR_STAYDOWNTIME = 30.0f;

const kStr MAP_SHOCK_FX = "fx/generic_217.kfx";
const kStr MAP_EXPLOSION_FX = "fx/generic_235.kfx";
const kStr MAP_EXPLOSION_SOUND1 = "sounds/shaders/generic_218.ksnd";
const kStr MAP_EXPLOSION_SOUND2 = "sounds/shaders/explosion_2.ksnd";
const array<kStr> gibNames = {Q::ACTOR_NAME_GIB1, Q::ACTOR_NAME_GIB2, Q::ACTOR_NAME_GIB3, Q::ACTOR_NAME_GIB4, Q::ACTOR_NAME_GIB5, Q::ACTOR_NAME_GIB6, Q::ACTOR_NAME_GIB7, Q::ACTOR_NAME_GIB8};

const int SCRIPT_TICK_CANNON_MOVE = 2;
const int SCRIPT_TICK_CANNON_EXPLOSIONS = 3;
const int SCRIPT_TICK_VICTORY_GIBS = 4;

enum EnumMapBossStages
{
	MAP_BOSS_STAGE_START = 0,
	MAP_BOSS_STAGE_SHEILD,
	MAP_BOSS_STAGE_CAMP,
	MAP_BOSS_STAGE_COMPLETE,
}

kActor@ campaigner;
kActor@ campDummyHealth;
kActor@ campFusionCannon;
kActor@ pillarLeft;
kActor@ pillarRight;
kActor@ shockPoint;
array<kActor@> gibPoints;
array<kActor@> campaignerWarpPoints;
float campNoSeeTime;
bool campTeleport;
float campTeleportTime;
float campForceTeleportTime;
float campSpeedScale; //less is faster for animation speed
float campTimeSpeedScale; //less is faster for teleport timings
int campStartHealth;
bool showingBossBar;
float cannonMoveTime;
float cannonStartZ;
float cannonTargetZ;
bool cannonIsUp;
bool cannonTargetIsUp;
int bossStage;
int campaignerFightCount;
float pillarLeftDownTime;
float pillarRightDownTime;
bool isShockingCenter;
float shockingTime;
float cannonExplodeTime;
//------------------------------------------------------------------------------------------------------------------------
// OnStart
//------------------------------------------------------------------------------------------------------------------------
$script 0
{
	Q::OnMapStart(instigator);	
	Q::totalSecrets = 0;
	Q::totalEnemies = 29;
	Q::camExitPosActors.insertLast(World.GetActorByTID(TAG_CAMERA_MAPEXIT_POS));
	Q::camExitLookActors.insertLast(World.GetActorByTID(TAG_CAMERA_MAPEXIT_LOOK));
	if (!Player.HasWeapon(TW_WEAPON_MISSILE))
		Player.GiveWeapon(TW_WEAPON_MISSILE, 12);
	
	gibPoints = BP::Actor::Find(TAG_VICTORYGIB1, TAG_VICTORYGIB_FINAL);
	@shockPoint = World.GetActorByTID(TAG_SHOCK_POINT);
	@pillarLeft = World.GetActorByTID(TAG_PILLAR_LEFT);
	if (BP::Actor::Exists(pillarLeft))
	{
		pillarLeft.AnimState().Set(anim_doorClose, MAP_PILLAR_ANIMSPEED, ANF_CYCLECOMPLETED);
		pillarLeft.AnimState().SetLastFrame();
	}
	@pillarRight = World.GetActorByTID(TAG_PILLAR_RIGHT);
	if (BP::Actor::Exists(pillarRight))
	{
		pillarRight.AnimState().Set(anim_doorClose, MAP_PILLAR_ANIMSPEED, ANF_CYCLECOMPLETED);
		pillarRight.AnimState().SetLastFrame();
	}
	@campFusionCannon = World.GetActorByTID(TAG_CAMPAIGNER_FUSION_CANNON);
	@campDummyHealth = World.GetActorByTID(TAG_CAMPAIGNER_DUMMY_HEALTH);
	@campaigner = World.GetActorByTID(TAG_AI_CAMPAIGNER);
	if (BP::Actor::Exists(campaigner) && !BP::Actor::IsDead(campaigner))
	{
		campStartHealth = campaigner.SpawnParams(2);
		if (BP::Actor::Exists(campDummyHealth))
			campDummyHealth.Health() = campStartHealth;
		kActor@ itemRune = World.GetActorByTID(TAG_ITEM_RUNE);
		if (!BP::Actor::Exists(itemRune))
		{
			SetBossStage(MAP_BOSS_STAGE_SHEILD);
		}
		else
		{
			SetBossStage(MAP_BOSS_STAGE_START);
		}
	}
	else
	{
		@campaigner = null;
		SetBossStage(MAP_BOSS_STAGE_COMPLETE);
		if (BP::Actor::Exists(campDummyHealth))
			campDummyHealth.Health() = 0;
	}
	for (int i = TAG_CAMPAIGNER_WP_START; i <= TAG_CAMPAIGNER_WP_FINISH; i++)
	{
		kActor@ actor = World.GetActorByTID(i);
		if (BP::Actor::Exists(actor))
			campaignerWarpPoints.insertLast(actor);
	}
	
	campSpeedScale = 1.0f;
	campTimeSpeedScale = 1.0f;
	
	Game.CallDelayedMapScript(SCRIPT_TICK_CANNON_MOVE, instigator, 0.0f);
	Game.CallDelayedMapScript(SCRIPT_TICK_CANNON_EXPLOSIONS, instigator, 0.0f);
}
//------------------------------------------------------------------------------------------------------------------------
// OnTick
//------------------------------------------------------------------------------------------------------------------------
$script 1
{
	Q::OnMapTick();
	if (campaigner !is null && campaigner.IsStale())
		@campaigner = null;
	TickCampaigner();
	TickPillars();
	TickShocking();
	TickBossStage();

    $restart;
}
//------------------------------------------------------------------------------------------------------------------------
// Tick Campaigner Fusion Cannon moving
//------------------------------------------------------------------------------------------------------------------------
$script 2
{
	if (BP::Actor::Exists(campFusionCannon) && cannonMoveTime > 0.0f)
	{
		cannonMoveTime -= GAME_DELTA_TIME;
		float t = cannonMoveTime / MAP_CFC_MOVETIME;
		campFusionCannon.Origin().z = BP::Math::SmoothStepFloat(cannonTargetZ, cannonStartZ, t);
		if (t <= 0.0f)
		{
			cannonIsUp = cannonTargetIsUp;
			cannonMoveTime = 0.0f;
		}
	}
	
	$restart;
}
//------------------------------------------------------------------------------------------------------------------------
// Explosions on Campaigners Fusion Cannon
//------------------------------------------------------------------------------------------------------------------------
$script 3
{
	if (!BP::Actor::Exists(campFusionCannon))
		return;
	
	if (cannonExplodeTime > 0.0f && (cannonIsUp || cannonMoveTime > 0.0f))
	{
		cannonExplodeTime -= GAME_DELTA_TIME;
		
		kActor@ soundActor = null;
		if (campaignerFightCount < 2)
			@soundActor = @BP::playerActor;
		else if (BP::Actor::Exists(shockPoint))
			@soundActor = @shockPoint;
		else
			@soundActor = @campFusionCannon;
		
		float x, y, z;
		for(int i = 0; i < 2; ++i)
		{
			float r = campFusionCannon.Radius();
			float h = campFusionCannon.Height();
			x = Math::RandRange(-r, r);
			y = Math::RandRange(-r, r);
			z = Math::RandRange(0.0f, h);
			
			kVec3 exPos = campFusionCannon.Origin() + kVec3(x, y, z);
			Game.SpawnFx(MAP_EXPLOSION_FX, exPos, 0);
		}
		
		if ((Math::Rand() % -2) != 0)
			soundActor.PlaySound(MAP_EXPLOSION_SOUND1);
		else
			soundActor.PlaySound(MAP_EXPLOSION_SOUND2);
		
		float delayTime = 0.125f;
		if (campaignerFightCount >= 2)
			delayTime = 0.22f;
		
		delay(delayTime);
		cannonExplodeTime -= delayTime;
    }
	
    $restart;
}
//------------------------------------------------------------------------------------------------------------------------
// Victory Gibs
//------------------------------------------------------------------------------------------------------------------------
$script 4
{
	int sectorIndex;
	kVec3 gibPos;

	for (uint i = 0; i < gibPoints.length(); i++)
	{
		gibPos = gibPoints[i].Origin();
		sectorIndex = gibPoints[i].SectorIndex();
		
		int gibCount = 2 + Math::RandMax(3);
		for (int i2 = 0; i2 < gibCount; i2++)
		{
			int gibIndex = BP::Array::RandomIndex(gibNames.length());
			kActor@ gib = BP::Actor::Spawn(gibNames[gibIndex], gibPos, 0.0f, sectorIndex);
			if (BP::Actor::Exists(gib))
			{
				gib.Flags() |= AF_IGNORESOUNDEVENTS;
				BP::Actor::TossItem(gib, 2.0f);
			}
		}
	}
	
	delay(Math::RandRange(0.125f, 0.25f));
    $restart;
}
//------------------------------------------------------------------------------------------------------------------------
void StartFusionCannonUp()
{
	cannonStartZ = BP::Actor::Exists(campFusionCannon) ? campFusionCannon.Origin().z : MAP_CFC_DOWNLERP_Z;
	cannonTargetZ = MAP_CFC_UP_Z;
	cannonTargetIsUp = true;
	cannonMoveTime = MAP_CFC_MOVETIME;
}
//------------------------------------------------------------------------------------------------------------------------
void StartFusionCannonDown()
{
	cannonStartZ = BP::Actor::Exists(campFusionCannon) ? campFusionCannon.Origin().z : MAP_CFC_UP_Z;
	cannonTargetZ = MAP_CFC_DOWN_Z;
	cannonTargetIsUp = false;
	cannonMoveTime = MAP_CFC_MOVETIME;
}
//------------------------------------------------------------------------------------------------------------------------
void SetBossStage(const int stage)
{
	int lastBossStage = bossStage;
	bossStage = stage;
	switch (bossStage)
	{
		case MAP_BOSS_STAGE_START:
		{
			campaignerFightCount = 0;
			if (BP::Actor::Exists(campaigner))
			{
				campaigner.ImpactType() = IT_FORCEFIELD;
			}
			if (BP::Actor::Exists(campFusionCannon))
			{
				campFusionCannon.Origin().z = MAP_CFC_DOWN_Z;
				cannonIsUp = false;
			}
			break;
		}
		case MAP_BOSS_STAGE_SHEILD:
		{
			if (lastBossStage == MAP_BOSS_STAGE_CAMP)
				campForceTeleportTime = MAP_CAMPAIGNER_FORCE_TELEPORT_TIME;
			
			StartFusionCannonUp();
			if (BP::Actor::Exists(campaigner))
			{
				campaigner.ImpactType() = IT_FORCEFIELD;
			}
			break;
		}
		case MAP_BOSS_STAGE_CAMP:
		{
			if (campaignerFightCount == 0)
				World.TriggerActorsByTID(BP::playerActor, TAG_SHOW_ENEMIES2);
			else if (campaignerFightCount == 1)
				World.TriggerActorsByTID(BP::playerActor, TAG_SHOW_ENEMIES4);
			else if (campaignerFightCount == 2)
				World.TriggerActorsByTID(BP::playerActor, TAG_SHOW_ENEMIES6);
				
			cannonExplodeTime = MAP_CFC_EXPLODETIME;
			for (int i = 0; i < int(campaignerWarpPoints.length()); i++)
			{
				BP::Game::AddShake(campaignerWarpPoints[i], MAP_CFC_EXPLODETIME);
			}
			
			if (BP::Actor::Exists(campaigner))
				campaigner.ImpactType() = IT_FLESH_HUMAN;
			
			if (campaignerFightCount >= 2)
				StartFusionCannonUp();
			else
				StartFusionCannonDown();
			
			break;
		}
		case MAP_BOSS_STAGE_COMPLETE:
		{
			campaignerFightCount = 3;
			if (BP::Actor::Exists(campaigner))
				campaigner.ImpactType() = IT_FLESH_HUMAN;
			StartFusionCannonDown();
			break;
		}
	}
}
//------------------------------------------------------------------------------------------------------------------------
void TickBossStage()
{
	switch (bossStage)
	{
		case MAP_BOSS_STAGE_START:
		{
			break;
		}
		case MAP_BOSS_STAGE_SHEILD:
		{
			break;
		}
		case MAP_BOSS_STAGE_CAMP:
		{
			int health = BP::Actor::Exists(campDummyHealth) ? campDummyHealth.Health() : 0;
			int nextStage = MAP_BOSS_STAGE_CAMP;
			if (campaignerFightCount == 0 && health <= int(float(campStartHealth) * 0.66f))
			{
				World.TriggerActorsByTID(BP::playerActor, TAG_SHOW_ENEMIES3);
				nextStage = MAP_BOSS_STAGE_SHEILD;
				campaignerFightCount++;
				if (Game.GetDifficulty() >= DIFFICULTY_HARD)
					campSpeedScale -= 0.17f;
				campTimeSpeedScale -= 0.25f;
			}
			if (campaignerFightCount == 1 && health <= int(float(campStartHealth) * 0.33f))
			{
				World.TriggerActorsByTID(BP::playerActor, TAG_SHOW_ENEMIES5);
				nextStage = MAP_BOSS_STAGE_SHEILD;
				campaignerFightCount++;
				if (Game.GetDifficulty() >= DIFFICULTY_HARD)
					campSpeedScale -= 0.17f;
				campTimeSpeedScale -= 0.25f;
			}
			if (campaignerFightCount >= 2 && health <= 0)
			{
				nextStage = MAP_BOSS_STAGE_COMPLETE;
				campaignerFightCount++;
				if (Game.GetDifficulty() >= DIFFICULTY_HARD)
				{
					campSpeedScale -= 0.20f;
					campTimeSpeedScale -= 0.25f;
				}
			}
			
			if (nextStage != MAP_BOSS_STAGE_CAMP)
				SetBossStage(nextStage);
			
			break;
		}
		case MAP_BOSS_STAGE_COMPLETE:
		{
			break;
		}
	}
}
//------------------------------------------------------------------------------------------------------------------------
void TickCampaigner()
{
	if (!IsCampaignerActive())
	{
		if (showingBossBar)
		{
			showingBossBar = false;
			PlayLoop.RemoveBossActor();
		}
		return;
	}
	
	if (!showingBossBar)
	{
		if (BP::Actor::Exists(campDummyHealth))
			PlayLoop.TagActorForBossBar(campDummyHealth);
		showingBossBar = true;
	}
	
	int animID = campaigner.AnimState().PlayingID();
	int areaID = campaigner.AreaID();
	int sectorIndex = campaigner.SectorIndex();
	int areaFlags = World.GetAreaFlags(areaID);
	
	if (animID == anim_aiTeleportOut)
	{
		campForceTeleportTime = 0.0f;
	}
	
	if (campSpeedScale != 1.0f)
	{
		campaigner.AnimState().ChangeSpeed(2.6f * campSpeedScale);
	}
	
	//check if needs to teleport
	if (campTeleport)
	{
		campTeleportTime += GAME_DELTA_TIME;
		if (campTeleportTime >= MAP_CAMPAIGNER_TELEPORT_TIME * campTimeSpeedScale)
		{
			campTeleport = false;
			if (campaignerWarpPoints.length() > 0)
			{
				int warpIndex = BP::Array::RandomIndex(campaignerWarpPoints.length());
				kActor@ warpPoint = campaignerWarpPoints[warpIndex];
				campaigner.PlaySound("sounds/shaders/campainger_spell.ksnd");
				campaigner.RunFxEvent("Campaigner_TeleportIn");
				campaigner.Origin() = warpPoint.Origin();
				campaigner.SetSector(warpPoint.SectorIndex());
			}
		}
	}
	else
	{
		//if in invalid area then teleport
		if ((areaFlags & (AAF_WATER|AAF_BLOCK|AAF_CLIFF|AAF_RESTRICTED)) != 0)
		{
			CampaignerStartTeleport();
		}
		
		//if can't see player after so long then teleport
		if (!campaigner.CanSee(BP::playerActor))
		{
			campNoSeeTime += GAME_DELTA_TIME;
			if (campNoSeeTime >= MAP_CAMPAIGNER_NO_SEE_TELEPORT_TIME * campTimeSpeedScale)
			{
				CampaignerStartTeleport();
			}
		}
		else
		{
			campNoSeeTime = 0.0f;
		}
		
		//if hasn't teleported in a while then teleport
		campForceTeleportTime += GAME_DELTA_TIME;
		if (campForceTeleportTime >= MAP_CAMPAIGNER_FORCE_TELEPORT_TIME * campTimeSpeedScale)
		{
			CampaignerStartTeleport();
		}
	}
}
//------------------------------------------------------------------------------------------------------------------------
bool IsCampaignerActive()
{
	return (BP::Actor::Exists(campaigner) && !BP::Actor::IsDead(campaigner) && !BP::Actor::IsHidden(campaigner));
}
//------------------------------------------------------------------------------------------------------------------------
void CampaignerStartTeleport()
{
	if (!IsCampaignerActive())
		return;
	
	//not teleporting and not in stun and not in start/activate animation
	int animID = campaigner.AnimState().PlayingID();
	if (animID != anim_aiTeleportOut && animID != anim_campaingerCrumble && animID != anim_activate)
	{
		campTeleport = true;
		campTeleportTime = 0.0f;
		campForceTeleportTime = 0.0f;
		campNoSeeTime = 0.0f;
		campaigner.AnimState().Blend(anim_aiTeleportOut, 2.0f, 8.0f, ANF_ROOTMOTION);
		campaigner.PlaySound("sounds/shaders/campainger_spell.ksnd");
		campaigner.RunFxEvent("Campaigner_TeleportOut");
	}
}
//------------------------------------------------------------------------------------------------------------------------
void TickPillars()
{
	if (isShockingCenter)
		return;
	
	if (BP::Actor::Exists(pillarLeft))
	{
		if (pillarLeft.AnimState().PlayingID() == anim_doorOpen && pillarLeft.AnimState().CycleCompleted())
		{
			pillarLeftDownTime += GAME_DELTA_TIME;
			if (pillarLeftDownTime >= MAP_PILLAR_STAYDOWNTIME)
			{
				pillarLeftDownTime = 0.0f;
				pillarLeft.AnimState().Set(anim_doorClose, MAP_PILLAR_ANIMSPEED, 0);
			}
		}
	}
	
	if (BP::Actor::Exists(pillarRight))
	{
		if (pillarRight.AnimState().PlayingID() == anim_doorOpen && pillarRight.AnimState().CycleCompleted())
		{
			pillarRightDownTime += GAME_DELTA_TIME;
			if (pillarRightDownTime >= MAP_PILLAR_STAYDOWNTIME)
			{
				pillarRightDownTime = 0.0f;
				pillarRight.AnimState().Set(anim_doorClose, MAP_PILLAR_ANIMSPEED, 0);
			}
		}
	}
}
//------------------------------------------------------------------------------------------------------------------------
void TickShocking()
{
	if (isShockingCenter)
	{
		shockingTime -= GAME_DELTA_TIME;
		if (shockingTime <= 0.0f)
		{
			isShockingCenter = false;
			pillarLeftDownTime = 0.0f;
			pillarRightDownTime = 0.0f;
			pillarLeft.AnimState().Set(anim_doorClose, MAP_PILLAR_ANIMSPEED, 0);
			pillarRight.AnimState().Set(anim_doorClose, MAP_PILLAR_ANIMSPEED, 0);
		}
	}
	
	if (campaignerFightCount >= 2 && bossStage != MAP_BOSS_STAGE_SHEILD)
		cannonExplodeTime = MAP_CFC_EXPLODETIME;
}
//------------------------------------------------------------------------------------------------------------------------
// Map Exit
//------------------------------------------------------------------------------------------------------------------------
$script 7000
{
	if (BP::Actor::Exists(pillarLeft) && pillarLeft.AnimState().PlayingID() != anim_doorClose)
		pillarLeft.AnimState().Set(anim_doorClose, MAP_PILLAR_ANIMSPEED, 0);
	if (BP::Actor::Exists(pillarRight) && pillarRight.AnimState().PlayingID() != anim_doorClose)
		pillarRight.AnimState().Set(anim_doorClose, MAP_PILLAR_ANIMSPEED, 0);
	Game.CallDelayedMapScript(SCRIPT_TICK_VICTORY_GIBS, instigator, 0.0f);
	Q::NextMap();
}
//------------------------------------------------------------------------------------------------------------------------
// Left Button
//------------------------------------------------------------------------------------------------------------------------
$script 7001
{
	if (BP::Actor::Exists(pillarLeft))
	{
		if (pillarLeft.AnimState().PlayingID() == anim_doorClose && pillarLeft.AnimState().CycleCompleted())
		{
			pillarLeft.AnimState().Set(anim_doorOpen, MAP_PILLAR_ANIMSPEED, 0);
		}
	}
}
//------------------------------------------------------------------------------------------------------------------------
// Right Button
//------------------------------------------------------------------------------------------------------------------------
$script 7002
{
	if (BP::Actor::Exists(pillarRight))
	{
		if (pillarRight.AnimState().PlayingID() == anim_doorClose && pillarRight.AnimState().CycleCompleted())
		{
			pillarRight.AnimState().Set(anim_doorOpen, MAP_PILLAR_ANIMSPEED, 0);
		}
	}
}
//------------------------------------------------------------------------------------------------------------------------
// Center Button
//------------------------------------------------------------------------------------------------------------------------
$script 7003
{
	if (BP::Actor::Exists(pillarLeft) && BP::Actor::Exists(pillarRight) && !isShockingCenter)
	{
		if (pillarLeft.AnimState().PlayingID() == anim_doorOpen && pillarLeft.AnimState().CycleCompleted() &&
			pillarRight.AnimState().PlayingID() == anim_doorOpen && pillarRight.AnimState().CycleCompleted())
		{
			isShockingCenter = true;
			shockingTime = MAP_PILLAR_SHOCKTIME;
			if (BP::Actor::Exists(shockPoint))
				shockPoint.SpawnFx(MAP_SHOCK_FX, Math::vecZero, shockPoint.Rotation());
			
			if (bossStage == MAP_BOSS_STAGE_SHEILD && cannonIsUp)
			{
				SetBossStage(MAP_BOSS_STAGE_CAMP);
			}
		}
	}
}
//------------------------------------------------------------------------------------------------------------------------
// Picked up Rune
//------------------------------------------------------------------------------------------------------------------------
$script 7004
{
	if (bossStage == MAP_BOSS_STAGE_START)
		SetBossStage(MAP_BOSS_STAGE_SHEILD);
}
//------------------------------------------------------------------------------------------------------------------------
// Campaigner killed EventLink
//------------------------------------------------------------------------------------------------------------------------
$script 7005
{
	
}
//------------------------------------------------------------------------------------------------------------------------
