#include "scripts/Q/Q_Common_Map.cpp"

//TIDs
//1000-1020: Camera Move Points

const float CAM_MOVESPEED = 2.56f;
const float CAM_TURNSPEED = 0.01f;
const float POINT_RADIUS = 20.48f;

array<kActor@> movePoints;
int curMovePoint;
//------------------------------------------------------------------------------------------------------------------------
$script 0
{
	//New game is whenever player enters this map
	BP::NewGame();
	BP::GameVar::SetInt(Q::GV_KEY_CAMP_DIF, DIFFICULTY_NORMAL);
	BP::GameVar::SetFloat(Q::GV_KEY_MAP_TIME, 0.0f);
	BP::GameVar::SetFloat(Q::GV_KEY_MAP_EXITTIME, 0.0f);
	BP::GameVar::SetInt(Q::GV_KEY_MAP_SECRETS, 0);
	BP::GameVar::SetInt(Q::GV_KEY_MAP_KILLS, 0);
	BP::GameVar::SetString(Q::GV_KEY_MAPS, "");
	BP::GameVar::SetInt(Q::GV_KEY_GAME_FLAGS, 0);
	BP::GameVar::SetInt(Q::GV_KEY_INIT_DEATHCOUNT, 0);

	Q::OnMapStart(instigator);

	BP::Music::Play(Q::PATH_MUSIC_THEME);
	GameVariables.SetValue("g_startmap", Q::PATH_MAP_INTRO);
    GameVariables.SetValue("g_restartmap", Q::PATH_MAP_TITLE);
	
    bool bStartedIntro;
    GameVariables.GetBool("bStartedIntro", bStartedIntro);
	if (!bStartedIntro)
	{
		BP::Log("--------------------------------------------------------------------------------------------------------------------------------");
		BP::LogError("QTurok v" + Q::DEV_VERSION);
		BP::LogError("By: BehemothProgrammer");
		BP::Log("--------------------------------------------------------------------------------------------------------------------------------");
	}
	GameVariables.SetValue("bStartedIntro", "1");

	movePoints = BP::Actor::Find(1000, 1019);
    Camera.StartCinematic(CMF_LOCK_PLAYER|CMF_NO_LETTERBOX|CMF_NO_INITIAL_FADEOUT);
    Camera.fov = 90.0f;
	Camera.yaw = Math::Deg2Rad(90.0f);
	curMovePoint = 0;
	if (HasValidMovePoints())
		Camera.origin = movePoints[curMovePoint].Origin();
	else
		Camera.origin = Math::vecZero;
	
	Game.CallDelayedMapScript(Q::SCRIPT_MAP_COMMON_TICK, instigator, 0.0f);
	Game.CallDelayedMapScript(2, instigator, 0.0f);
}
//------------------------------------------------------------------------------------------------------------------------
// Map Tick
//------------------------------------------------------------------------------------------------------------------------
$script 1
{
	if (HasValidMovePoints())
	{
		int nextMovePointIndex = curMovePoint + 1;
		if (nextMovePointIndex >= int(movePoints.length()))
			nextMovePointIndex = 0;
			
		kActor@ nextMovePoint = movePoints[nextMovePointIndex];
		kVec3 moveDir = (nextMovePoint.Origin() - Camera.origin).Normalize();
		Camera.origin += moveDir * CAM_MOVESPEED;
		Camera.yaw = Camera.yaw.Interpolate(moveDir.ToYaw(), CAM_TURNSPEED);
		Camera.pitch = Camera.pitch.Interpolate(-moveDir.ToPitch(), CAM_TURNSPEED);
		if (BP::Actor::IsInRange(nextMovePoint, Camera.origin, POINT_RADIUS))
		{
			curMovePoint++;
			if (curMovePoint >= int(movePoints.length()))
				curMovePoint = 0;
		}
	}
	
    $restart;
}
//------------------------------------------------------------------------------------------------------------------------
$script 2
{
	if (Camera.UserInterrupted())
	{
		Game.ShowMainMenu();
		return;
	}
	
    $restart;
}
//------------------------------------------------------------------------------------------------------------------------
bool HasValidMovePoints()
{
	return movePoints.length() > 1;
}
//------------------------------------------------------------------------------------------------------------------------
