#include "BP_Addons/scripts/BP_Common_Map.cpp"
#include "scripts/Q/Q_Common_Shared.cpp"

namespace Q
{
	const uint16 OPEN_MENU_BUTTONS = (BC_MAPZOOMIN|BC_MAPZOOMOUT);
	const int TAG_ENEMIES_FIRST = 1500;
	const int TAG_ENEMIES_LAST = 2499;
	const int SCRIPT_CINEMA_MAP_RESULTS = 51000;

	kActor@ map;
	kStr nextMapPath;
	int totalEnemies;
	int totalSecrets;
	BP::CL_BP_ActorIterator aiIter;
	array<kActor@> camExitPosActors;
	array<kActor@> camExitLookActors;

	//------------------------------------------------------------------------------------------------------------------------
	void OnMapStart(kActor@ instigator)
	{
		BP::MapStart();
		
		nextMapPath = "";
		totalEnemies = 0;
		totalSecrets = 0;
		
		float mapTime;
		int mapSecrets;
		int mapKills;
		GetMapData(Game.GetCurrentMapID(), mapTime, mapSecrets, mapKills);
		BP::GameVar::SetFloat(GV_KEY_MAP_TIME, mapTime);
		BP::GameVar::SetInt(GV_KEY_MAP_SECRETS, mapSecrets);
		BP::GameVar::SetInt(GV_KEY_MAP_KILLS, mapKills);
		BP::GameVar::SetFloat(GV_KEY_MAP_EXITTIME, 0.0f);

		bool isQPlayer = BP::Actor::GetDefBool(BP::playerActor, DEF_PLAYER_EXISTS);
		
		float aiHealthScale = 1.0f;
		switch (Game.GetDifficulty())
		{
			case DIFFICULTY_HARDCORE:
			{
				aiHealthScale = 1.25f;
				break;
			}
		}
		
		//Setup AI
		aiIter.Setup(10, TickAI);
		for (int i = TAG_ENEMIES_FIRST; i <= TAG_ENEMIES_LAST; i ++)
		{
			kActor@ actor = World.GetActorByTID(i);
			if (BP::Actor::Exists(actor) && !actor.IsPersistentMarked())
			{
				if (!AICanSpawn(actor))
				{
					actor.Remove();
				}
				else
				{
					aiIter.AddActor(actor);
					actor.Health() = int(actor.Health() * aiHealthScale);
					if (AIStartsHidden(actor))
						BP::Actor::HideAI(actor);
				}
			}
		}
		
		//if not at start up map
		if (Game.GetCurrentMapID() != MAP_TUROK_START)
		{
			bool hasGameStarted = BP::GameVar::GetIntFlag(GV_KEY_GAME_FLAGS, GF_GAMESTARTED);
			if (!hasGameStarted)
			{
				if (Game.GetCurrentMapID() != MAP_HUB)
				{
					Player.Lock();
					Game.ChangeMap(PATH_MAP_INTRO);
					Game.HaltMapScript(SCRIPT_MAP_COMMON_START);
					BP::Wait(); //Need to wait to make this script no longer continue to execute
				}
				
				BP::GameVar::SetIntFlag(GV_KEY_GAME_FLAGS, GF_GAMESTARTED);
			}
			
			int initDeathCount = BP::GameVar::GetInt(GV_KEY_INIT_DEATHCOUNT);
			if (initDeathCount < 3)
			{
				BP::GameVar::SetInt(GV_KEY_INIT_DEATHCOUNT, initDeathCount + 1);
				Player.Lock();
				if (initDeathCount < 2)
				{
					PlayLoop.HandlePlayerDeath(true);
				}
				Game.ChangeMap(PATH_MAP_INTRO);
				Game.HaltMapScript(SCRIPT_MAP_COMMON_START);
				BP::Wait(); //Need to wait to make this script no longer continue to execute
			}
			
			BP::playerActor.Gravity() = PLAYER_GRAVITY;
			@map = BP::Actor::Spawn(AT_MAP, BP::playerActor);
			
			//Start common map tick
			Game.CallDelayedMapScript(SCRIPT_MAP_COMMON_TICK, instigator, 0.0f);
			
			if (!Player.HasWeapon(TW_WEAPON_SHOTGUN))
				Player.GiveWeapon(TW_WEAPON_SHOTGUN, PLAYER_STARTSHELLAMMO);
			
			//if a cinematic is already playing then wait for it to finish (e.g. the death cinematic)
			BP::WaitUntil(BP::Cinema::IsFinished);
		}
	}
	//------------------------------------------------------------------------------------------------------------------------
	void OnMapTick()
	{
		aiIter.Tick();
		BP::Input::Tick();
		
		int areaID = BP::playerActor.AreaID();
		uint playerAreaFlags = World.GetAreaFlags(areaID);
		bool playerIsGrounded = BP::playerActor.OnGround();
		bool playerInWater = BP::Actor::IsInWater(BP::playerActor);
		
		if (!playerInWater && (playerAreaFlags & (AAF_DEATHPIT|AAF_CLIFF) == 0) && BP::playerPuppet.PlayerFlags() & PF_NOAIRFRICTION == 0)
		{
			//prevent on ground then suddenly alot of velocity to the ground
			if (playerIsGrounded && BP::playerActor.Velocity().z < 0.0f)
			{
				BP::playerActor.Velocity().z = 0.0f;
			}
		
			//if will be grounded on next tick then stop z velocity to always allow soft landing (preventing the movement stop on landing)
			bool willBeGrounded = !playerIsGrounded && BP::playerActor.Velocity().z < 0.0f && (BP::playerActor.Origin().z + (BP::playerActor.Velocity().z + BP::playerActor.Gravity() - 2.56f) <= BP::playerActor.FloorHeight());
			if (willBeGrounded)
			{
				if (BP::playerActor.Velocity().z <= -15.0)
				{
					BP::playerActor.PlaySound("sounds/shaders/generic_22_turok_land.ksnd");
				}
				BP::playerActor.Velocity().z = 0.0f;
				BP::playerActor.Origin().z = BP::playerActor.FloorHeight();
			}
		}
		
		if (Game.GetCurrentMapID() == MAP_HUB)
			return;
		
		//Make it so the player can not use the bow (because of 1 hit super shot) on Hard or Hardcore difficulty
		if (Game.GetDifficulty() >= DIFFICULTY_HARD)
		{
			int bowAmmo = Player.GetAmmo(TW_WEAPON_BOW);
			if (Player.CurrentWeapon() == TW_WEAPON_BOW && bowAmmo > 0 && Player.GetAltAmmo(TW_WEAPON_BOW) <= 0)
			{
				Player.ConsumeAmmo(bowAmmo);
			}
		}
		
		int campaignDifficulty = BP::GameVar::GetInt(GV_KEY_CAMP_DIF);
		if (Game.GetDifficulty() != campaignDifficulty)
		{
			BP::GameVar::SetInt(GV_KEY_DIFFICULTY, campaignDifficulty);
			Player.Lock();
			BP::playerActor.PlaySound("sounds/shaders/generic_25_turok_fall_to_death.ksnd");
			BP::HUD::ShowHelp(BP::Game::GetLocalText(13), 300); //can't change difficulty
			BP::Wait(3.0f);
			PlayLoop.HandlePlayerDeath(true);
		}
		
		if (!Player.Locked() && !BP::Cinema::IsActive() && BP::Input::Pressing(OPEN_MENU_BUTTONS))
		{
			float mapTime = BP::GameVar::GetFloat(GV_KEY_MAP_TIME); //time from the start of the map
			int secretsMask = BP::GameVar::GetInt(GV_KEY_MAP_SECRETS);
			int secretsFound = BP::Math::Min(BP::Math::BitCount(secretsMask, 0, 32), totalSecrets);
			int kills = BP::Math::Min(BP::GameVar::GetInt(GV_KEY_MAP_KILLS), totalEnemies);
			
			kStr timeString = BP::String::TimeFormatMMSS(mapTime + BP::Time::MapTime());
			kStr separator = "   ";
			BP::HUD::ShowHelp("Time " + timeString + separator + "Secrets " + secretsFound + " / " + totalSecrets + separator + "Kills " + kills + " / " + totalEnemies, 5);
		}
	}
	//------------------------------------------------------------------------------------------------------------------------
	bool TickAI(kActor@ actor)
	{
		if (actor.Type() == AT_AI_ZOMBIE)
			return true;
		
		bool exists = BP::Actor::Exists(actor);
		if (exists)
		{
			if (actor.Flags() & AF_DEAD != 0)
			{
				BP::GameVar::SetInt(GV_KEY_MAP_KILLS, BP::GameVar::GetInt(GV_KEY_MAP_KILLS) + 1);
				actor.ClipFlags() |= CF_DROPOFF;
				if (actor.Type() == AT_GRUNT)
				{
					if (actor.ModelVariation() <= GV_POACHER_SHOTGUN)
					{
						kActor@ drop = BP::Actor::Spawn(AT_DROP_SHELLS, actor);
						BP::Actor::TossItem(drop, 2.0f);
					}
				}
				else if (actor.Type() == AT_PURLIN)
				{
					kActor@ drop = BP::Actor::Spawn(AT_PICKUP_GRENADE, actor);
					BP::Actor::TossItem(drop, 2.0f);
				}
			}
		}
		
		return !exists || (actor.Flags() & AF_DEAD != 0);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void NextMap(const kStr &in mapPath = "")
	{
		nextMapPath = mapPath;
		if (BP::Actor::EnteredArea(BP::playerActor))
			BP::Cinema::StartScript(BP::playerActor, SCRIPT_CINEMA_MAP_RESULTS);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void OnMapExitCinemaEnd(const bool userInterrupted)
	{
		if (BP::String::IsEmpty(nextMapPath))
			Game.Restart();
		else
			Game.ChangeMap(nextMapPath);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void UpdateCompleteStatLines(array<kStr>& lines, const bool updateOnlyKills)
	{
		float mapTime = BP::GameVar::GetFloat(GV_KEY_MAP_TIME); //time from the start of the map
		int secretsMask = BP::GameVar::GetInt(GV_KEY_MAP_SECRETS);
		int secretsFound = BP::Math::BitCount(secretsMask, 0, 32);
		int kills = BP::GameVar::GetInt(GV_KEY_MAP_KILLS);
		
		float mapTotalTime = mapTime + BP::Time::MapTime();
		BP::GameVar::SetFloat(GV_KEY_MAP_EXITTIME, mapTotalTime);
		kStr timeString = "Time      " + BP::String::TimeFormatMMSS(mapTotalTime);
		kStr secretsString = "Secrets " + BP::String::PadLeft("" + BP::Math::Min(secretsFound, totalSecrets), 3, " ") + "/" + BP::String::PadLeft("" + BP::Math::Min(totalSecrets, 999), 3, " ");
		kStr killsString = "Kills   " + BP::String::PadLeft("" + BP::Math::Min(kills, totalEnemies), 3, " ") + "/" + BP::String::PadLeft("" + BP::Math::Min(totalEnemies, 999), 3, " ");
		int highestLength = BP::Math::Max(BP::String::Length(killsString), BP::Math::Max( BP::String::Length(secretsString), BP::Math::Max(BP::String::Length(timeString), 0) ) );
		timeString = BP::String::PadRight(timeString, highestLength, " ");
		secretsString = BP::String::PadRight(secretsString, highestLength, " ");
		killsString = BP::String::PadRight(killsString, highestLength, " ");
		
		if (!updateOnlyKills)
		{
			lines[0] = timeString;
			lines[2] = secretsString;
		}
		lines[4] = killsString;
	}
	//------------------------------------------------------------------------------------------------------------------------
}

//------------------------------------------------------------------------------------------------------------------------
// Cinema Map Exit Results
//------------------------------------------------------------------------------------------------------------------------
$script 51000 //SCRIPT_CINEMA_MAP_RESULTS
{
	kStr uiFxName = Q::PATH_FX_UI_COMPLETED;
	bool stopGameMusic = true;
	if (Game.GetCurrentMapID() == Q::MAP_E1M7)
	{
		uiFxName = Q::PATH_FX_UI_CONGRATULATIONS;
		stopGameMusic = false;
		Game.PlayMusic("music/track14.ogg", true);
	}
	else
	{
		BP::Music::Play(Q::PATH_MUSIC_TRACK03);
	}
	BP::Music::StopOnNextMap();
	
	BP::Cinema::Start(instigator, Q::SCRIPT_CINEMA_MAP_RESULTS, Q::OnMapExitCinemaEnd, CMF_LOCK_PLAYER|CMF_UNLOCK_PLAYER_ON_FINISH|CMF_NO_INITIAL_FADEOUT|CMF_NO_LETTERBOX);
	if (Q::camExitLookActors.length() > 0)
		BP::Cinema::LookTarget(Q::camExitLookActors[BP::Array::RandomIndex(Q::camExitLookActors.length())]);
	
	if (Q::camExitPosActors.length() > 0)
		BP::Cinema::CameraPos(Q::camExitPosActors[BP::Array::RandomIndex(Q::camExitPosActors.length())].Origin());
	else
		BP::Cinema::CameraPos(BP::playerActor.Origin());
	
	
	int kills = BP::GameVar::GetInt(Q::GV_KEY_MAP_KILLS);
	int linesLength = 8;
	array<kStr> lines(linesLength);
	Q::UpdateCompleteStatLines(lines, false);
	
	while (true)
	{
		if (stopGameMusic)
			Game.StopMusic();
		BP::playerActor.SpawnFx(uiFxName, Math::vecZero);
		int curKills = BP::GameVar::GetInt(Q::GV_KEY_MAP_KILLS);
		if (kills != curKills)
		{
			kills = curKills;
			Q::UpdateCompleteStatLines(lines, false);
		}
		for (int i = 0; i < linesLength; i++)
			Game.PrintLine(lines[(linesLength - 1) - i], i, 10);
		BP::Wait();
	}
	
	BP::Cinema::Stop();
}
