#include "scripts/common.txt"

namespace Q
{
	const bool DEV_MODE = true; //Currently not used
	const kStr DEV_VERSION = "1.0.1"; //Current version of QTurok (currently only used on map 42, map_title, OnStart to display in the console)

	const kStr GV_KEY_DIFFICULTY = "g_difficulty";
	const kStr GV_KEY_NEWGAME = "g_newgame";
	const kStr GV_KEY_SYSTEM = "Q_System";
	const kStr GV_KEY_CAMP_DIF = "Q_CampDif";
	const kStr GV_KEY_MAP_TIME = "Q_MapTime";
	const kStr GV_KEY_MAP_EXITTIME = "Q_MapExitTime";
	const kStr GV_KEY_MAP_SECRETS = "Q_MapSecrets";
	const kStr GV_KEY_MAP_KILLS = "Q_MapKills";
	const kStr GV_KEY_MAPS = "Q_Maps"; //stores: mapID, time, secrets, kills. For all maps.
	const kStr GV_KEY_GAME_FLAGS = "Q_GameFlags";
	const kStr GV_KEY_INIT_DEATHCOUNT = "Q_InitDeathCount";

	const float PLAYER_GRAVITY = 0.56f;
	const int PLAYER_STARTSHELLAMMO = 25;

	const kStr DEF_PLAYER_EXISTS = "QPlayer.exists";

	const int SCRIPT_MAP_COMMON_START = 0;
	const int SCRIPT_MAP_COMMON_TICK = 1;

	const kStr ACTOR_NAME_GIB1 = "Gib_Alien_Head";
	const kStr ACTOR_NAME_GIB2 = "Gib_Alien_Torso";
	const kStr ACTOR_NAME_GIB3 = "Gib_Alien_Feet";
	const kStr ACTOR_NAME_GIB4 = "Gib_Alien_Body";
	const kStr ACTOR_NAME_GIB5 = "Gib_Stalker_Head";
	const kStr ACTOR_NAME_GIB6 = "Gib_Stalker_Torso";
	const kStr ACTOR_NAME_GIB7 = "Gib_Stalker_Feet";
	const kStr ACTOR_NAME_GIB8 = "Gib_Stalker_Body";

	const kStr PATH_FX_UI_COMPLETED = "fx/quake/ui/completed.kfx";
	const kStr PATH_FX_UI_CONGRATULATIONS = "fx/quake/ui/congratulations.kfx";
	const kStr PATH_FX_GIBSPAWN = "fx/generic_108.kfx";

	const kStr PATH_SOUND_SUIT = "sounds/quake/items/suit.ksnd";
	const kStr PATH_SOUND_SUIT2 = "sounds/quake/items/suit2.ksnd";

	const kStr PATH_MAP_TITLE = "levels/level42.map";
	const kStr PATH_MAP_INTRO = "levels/quake/intro.map";
	const kStr PATH_MAP_E1M1 = "levels/quake/e1m1.map";
	const kStr PATH_MAP_E1M2 = "levels/quake/e1m2.map";
	const kStr PATH_MAP_E1M3 = "levels/quake/e1m3.map";
	const kStr PATH_MAP_E1M7 = "levels/quake/e1m7.map";

	const kStr PATH_MUSIC_THEME = "sounds/quake/music/theme.ksnd";
	const kStr PATH_MUSIC_TRACK03 = "sounds/quake/music/track03.ksnd";
	const kStr PATH_MUSIC_TRACK04 = "sounds/quake/music/track04.ksnd";
	const kStr PATH_MUSIC_TRACK06 = "sounds/quake/music/track06.ksnd";
	const kStr PATH_MUSIC_TRACK08 = "sounds/quake/music/track08.ksnd";
	const kStr PATH_MUSIC_TRACK09 = "sounds/quake/music/track09.ksnd";

	enum MapIDs
	{
		MAP_TUROK_START = 42,
		
		MAP_HUB = 64,
		MAP_E1M1 = 65,
		MAP_E1M2 = 66,
		MAP_E1M3 = 67,
		MAP_E1M7 = 71,
	}

	enum MapSavePointIDs //saves and checkpoints
	{
		SAVETID_HUB = 2601,
		SAVETID_E1M1 = 2602,
		SAVETID_E1M2 = 2603,
	}

	enum ActorTypes
	{
		AT_MAP = 6001,
		AT_BIOSUIT = 6002,
		AT_DROP_SHELLS = 6003,
		AT_DROP_INVISIBLEBACKPACK = 6004,
		AT_AI_FLYING_MANTIS = 6005,
		AT_AI_ZOMBIE = 6006,
	}

	// are not reset on newgame
	enum SystemFlags
	{
		SF_EASYFINISHED = (1 << 0),
		SF_NORMALFINISHED = (1 << 1),
		SF_HARDFINISHED = (1 << 2),
		SF_HARDCOREFINISHED = (1 << 3)
	}

	enum GameFlags
	{
		GF_GAMESTARTED = (1 << 0)
	}

	enum EnemySpawnFlags
	{
		AI_FLAG_STARTHIDDEN = (1 << 0),
		AI_FLAG_NOEASY = (1 << 1), //does NOT spawn in easy difficulty
		AI_FLAG_NONORMAL = (1 << 2), //does NOT spawn in normal difficulty
		AI_FLAG_NOHARD = (1 << 3), //does NOT spawn in hard difficulty
		AI_FLAG_NOHARDCORE = (1 << 4), //does NOT spawn in hardcore difficulty
	}

	//------------------------------------------------------------------------------------------------------------------------
	bool AIStartsHidden(kActor@ actor)
	{
		return (actor.SpawnParams(7) & AI_FLAG_STARTHIDDEN != 0);
	}
	//------------------------------------------------------------------------------------------------------------------------
	bool AICanSpawn(kActor@ actor)
	{
		int spawnFlags = actor.SpawnParams(7);
		int difficulty = Game.GetDifficulty();
		return ((difficulty == DIFFICULTY_EASY && spawnFlags & AI_FLAG_NOEASY == 0) ||
				(difficulty == DIFFICULTY_NORMAL && spawnFlags & AI_FLAG_NONORMAL == 0) ||
				(difficulty == DIFFICULTY_HARD && spawnFlags & AI_FLAG_NOHARD == 0) ||
				(difficulty == DIFFICULTY_HARDCORE && spawnFlags & AI_FLAG_NOHARDCORE == 0));
	}
	//------------------------------------------------------------------------------------------------------------------------
	// Finds a secret in the current map
	//------------------------------------------------------------------------------------------------------------------------
	void FindSecret(const int index)
	{
		int secrets = BP::GameVar::GetInt(GV_KEY_MAP_SECRETS);
		int secretFlag = (1 << index);
		if (secrets & secretFlag == 0)
		{
			secrets |= secretFlag;
			BP::GameVar::SetInt(GV_KEY_MAP_SECRETS, secrets);
			Game.PrintLine(BP::Game::GetLocalText(12), 0, 180); //Secret Found
			BP::playerActor.PlaySound(BP::Game::GetCustomSound(4));
		}
	}
	//------------------------------------------------------------------------------------------------------------------------
	bool GetMapData(const int mapID, float &out time, int &out secrets, int &out kills)
	{
		array<int> mapDataIDs;
		array<float> mapDataTimes;
		array<int> mapDataSecrets;
		array<int> mapDataKills;
		GetMapData(mapDataIDs, mapDataTimes, mapDataSecrets, mapDataKills);
		int index = mapDataIDs.find(mapID);
		if (index < 0)
		{
			time = 0.0f;
			secrets = 0;
			kills = 0;
			return false;
		}
		
		time = mapDataTimes[index];
		secrets = mapDataSecrets[index];
		kills = mapDataKills[index];
		
		return true;
	}
	//------------------------------------------------------------------------------------------------------------------------
	bool GetMapData(array<int> &out mapDataIDs, array<float> &out mapDataTimes, array<int> &out mapDataSecrets, array<int> &out mapDataKills)
	{
		array<kStr> dataAry = BP::String::Split(BP::GameVar::GetString(GV_KEY_MAPS), "|");
		if (dataAry.length() == 0)
			return false;
		
		mapDataIDs.resize(0);
		mapDataTimes.resize(0);
		mapDataSecrets.resize(0);
		mapDataKills.resize(0);
		for (uint i = 0; i < dataAry.length(); i+=4)
		{
			mapDataIDs.insertLast(dataAry[i].Atoi());
			mapDataTimes.insertLast(dataAry[i+1].Atof());
			mapDataSecrets.insertLast(dataAry[i+2].Atoi());
			mapDataKills.insertLast(dataAry[i+3].Atoi());
		}

		return true;
	}
	//------------------------------------------------------------------------------------------------------------------------
	void SetMapData(const int mapID, const float time, const int secrets, const int kills)
	{
		array<int> mapDataIDs;
		array<float> mapDataTimes;
		array<int> mapDataSecrets;
		array<int> mapDataKills;
		GetMapData(mapDataIDs, mapDataTimes, mapDataSecrets, mapDataKills);
		int index = mapDataIDs.find(mapID);
		if (index >= 0)
		{
			mapDataTimes[index] = time;
			mapDataSecrets[index] = secrets;
			mapDataKills[index] = kills;
		}
		else
		{
			mapDataIDs.insertLast(mapID);
			mapDataTimes.insertLast(time);
			mapDataSecrets.insertLast(secrets);
			mapDataKills.insertLast(kills);
		}
		
		kStr mapData = "" + mapDataIDs[0] + "|" + mapDataTimes[0] + "|" + mapDataSecrets[0] + "|" + mapDataKills[0];
		for (uint i = 1; i < mapDataIDs.length(); i++)
		{
			mapData += "|" + mapDataIDs[i] + "|" + mapDataTimes[i] + "|" + mapDataSecrets[i] + "|" + mapDataKills[i];
		}
		BP::GameVar::SetString(GV_KEY_MAPS, mapData);
	}
	//------------------------------------------------------------------------------------------------------------------------
	// On completing an episode you return to the hub with default weapons
	//------------------------------------------------------------------------------------------------------------------------
	void CompleteEpisode()
	{
		BP::GameVar::SetBool(GV_KEY_NEWGAME, true); //removes all weapons and reset lives
		BP::GameVar::SetInt(GV_KEY_INIT_DEATHCOUNT, 0);
		Game.ChangeMap(PATH_MAP_INTRO);
	}
	//------------------------------------------------------------------------------------------------------------------------
}
